@@ -5,33 +5,34 @@ import datetime
 import importlib
 import logging
 import time
+import uuid
+import json
 from abc import ABCMeta
-
+from flask import jsonify
 import requests
-
+from requests import RequestException
+from halo_flask.providers.providers import get_provider,ProviderError
 from .classes import AbsBaseClass
-from .exceptions import MaxTryHttpException, ApiError,NoApiDefinitionError
+from .exceptions import MaxTryHttpException, ApiError, NoApiDefinitionError, HaloMethodNotImplementedException,MissingClassConfigError,IllegalMethodException
 from .logs import log_json
 from .reflect import Reflect
-from .settingsx import settingsx
 from halo_flask.const import LOC,DEV,TST,PRD
 from .flask.utilx import Util
 from .const import HTTPChoice,SYSTEMChoice,LOGChoice
+from .settingsx import settingsx
 
 settings = settingsx()
 
-# DRF
-
-
+Rest = "rest"
+Soap = "soap"
+Rpc = "rpc"
 
 logger = logging.getLogger(__name__)
 
-
-
 from halo_flask.circuitbreaker import CircuitBreaker
 class MyCircuitBreaker(CircuitBreaker):
     def __init__(self):
-        print("init MyCircuitBreaker")
+        print("init MyCircuitBreaker:")
         FAILURE_THRESHOLD =  self.get_failure_threshold()
         RECOVERY_TIMEOUT = self.get_recovery_timeout()
         EXPECTED_EXCEPTION = Exception
@@ -49,6 +50,21 @@ class MyCircuitBreaker(CircuitBreaker):
             RECOVERY_TIMEOUT = settings.HTTP_RETRY_SLEEP
         return RECOVERY_TIMEOUT
 
+class AbsResponse(AbsBaseClass):
+    content = None
+    headers = None
+    status_code = None
+
+    def __init__(self, content, headers={},status_code=200):
+        self.content = content
+        self.headers = headers
+        self.status_code = status_code
+
+class SoapResponse(AbsResponse):
+    pass
+
+class ProviderResponse(SoapResponse):
+    pass
 
 class AbsBaseApi(AbsBaseClass):
     __metaclass__ = ABCMeta
@@ -57,27 +73,98 @@ class AbsBaseApi(AbsBaseClass):
     op = HTTPChoice.get.value
     url = None
     api_type = None
+    protocol = None
     halo_context = None
-    cb = MyCircuitBreaker()
+    #cb = MyCircuitBreaker()
+    session = None
+    version = None
 
-    def __init__(self, halo_context, method=None):
+    def __init__(self, halo_context, method=None,session=None):
         self.halo_context = halo_context
         if method:
             self.op = method
-        self.url, self.api_type = self.get_url_str()
-        if settings.CIRCUIT_BREAKER:
-            self.cb._name = self.name
+        self.url, self.api_type,self.protocol = self.get_url_str()
+        self.version = self.get_version()
+        if session:
+            self.session = session
+        else:
+            self.session = requests.session()
+        #if settings.CIRCUIT_BREAKER:
+            #self.cb._name = self.name
+
+    def get_url_str(self):
+        """
+
+        :return:
+        """
+        api_config = settings.API_CONFIG
+        logger.debug("api_config: " + str(api_config), extra=log_json(self.halo_context))
+        if api_config and self.name in api_config:
+            url = api_config[self.name]["url"]
+            type = api_config[self.name]["type"]
+            protocol = "rest"
+            if "protocol" in api_config[self.name]:
+                protocol = api_config[self.name]["protocol"]
+            return url,type,protocol
+
+        raise NoApiDefinitionError(self.name)
+
+    def get_version(self):
+        """
+
+        :return:
+        """
+        api_config = settings.API_CONFIG
+        version = '$LATEST'
+        if api_config and self.name in api_config:
+            if  "version" in api_config[self.name]:
+                version = api_config[self.name]["version"]
+        return version
+
+    def terminate(self):
+        if self.session:
+            print("terminate session:"+str(self.session))
+            self.session.close()
+            self.session = None
+
+    def provider_invoke(self,halo_context, method, url, api_type, timeout, data=None, headers=None, auth=None):
+
+        try:
+            service_name = self.name
+            logger.debug("send invoke to target_service:" + service_name, extra=log_json(halo_context))
+            if data:
+                x = str(data.decode('utf8').replace("'", '"'))
+                if x != "":
+                    datax = json.loads(x)
+                    datay = jsonify(datax)
+                else:
+                    datay = ""
+            else:
+                datay = ""
+            messageDict = {"method":method,"url":url,"data":datay,"headers":headers,"auth":auth}
+            print("messageDict="+str(messageDict)+" version:"+self.version)
+            ret = get_provider().invoke_sync(halo_context,messageDict,service_name,version=self.version)
+            print("ret:"+str(ret))
+            content = json.loads(ret['Payload'].read())
+            print("content:" + str(content))
+            provider_response = ProviderResponse(content["body"],ret['ResponseMetadata']["HTTPHeaders"],ret['StatusCode'])
+            return provider_response
+        except ProviderError as e:
+            logger.error("Unexpected Provider Error", extra=log_json(halo_context, messageDict, e))
+            raise e
+
+
+class AbsRestApi(AbsBaseApi):
+    __metaclass__ = ABCMeta
 
-    @cb
     def do_cb_request(self,method, url, timeout, data=None, headers=None, auth=None):
-        print("do MyCircuitBreaker")
-        return requests.request(method, url, data=data, headers=headers,
+        return self.session.request(method, url, data=data, headers=headers,
                          timeout=timeout, auth=auth)
 
     def do_request(self, method, url, timeout, data=None, headers=None, auth=None):
         if settings.CIRCUIT_BREAKER:
             return self.do_cb_request(method, url, timeout, data, headers, auth)
-        return requests.request(method, url, data=data, headers=headers,
+        return self.session.request(method, url, data=data, headers=headers,
                                 timeout=timeout, auth=auth)
 
     def exec_client(self, halo_context, method, url, api_type, timeout, data=None, headers=None, auth=None):
@@ -93,6 +180,9 @@ class AbsBaseApi(AbsBaseClass):
         :return:
         """
 
+        if api_type == 'service':
+            return self.provider_invoke(halo_context, method, url, api_type, timeout, data, headers, auth)
+
         msg = "Max Try for url: ("+str(settings.HTTP_MAX_RETRY)+") " + str(url)
         for i in range(0, settings.HTTP_MAX_RETRY):
             logger.debug("try index: " + str(i), extra=log_json(halo_context))
@@ -123,17 +213,6 @@ class AbsBaseApi(AbsBaseClass):
                 continue
         raise MaxTryHttpException(msg)
 
-    def get_url_str(self):
-        """
-
-        :return:
-        """
-        api_config = settings.API_CONFIG
-        logger.debug("api_config: " + str(api_config), extra=log_json(self.halo_context))
-        if api_config and self.name in api_config:
-            return api_config[self.name]["url"], api_config[self.name]["type"]
-        raise NoApiDefinitionError(self.name)
-
     def set_api_url(self, key, val):
         """
 
@@ -198,6 +277,24 @@ class AbsBaseApi(AbsBaseClass):
     def process(self, method, url, timeout, data=None, headers=None,auth=None):
         """
 
+        :param method:
+        :param url:
+        :param timeout:
+        :param data:
+        :param headers:
+        :return:
+        """
+        return self.process_rest(method, url, timeout, data, headers,auth)
+
+    def get_vars_from_url(self,url):
+        import urllib.parse as urlparse
+        from urllib.parse import parse_qs
+        parsed = urlparse.urlparse(url)
+        return parse_qs(parsed.query)
+
+    def process_rest(self, method, url, timeout, data=None, headers=None,auth=None):
+        """
+
         :param method:
         :param url:
         :param timeout:
@@ -324,16 +421,77 @@ class AbsBaseApi(AbsBaseClass):
             data = request.data
         return self.process(verb, self.url, Util.get_timeout(request), data=data, headers=headers,auth=auth)
 
+class AbsSoapApi(AbsBaseApi):
+    __metaclass__ = ABCMeta
+
+    def do_cb_request(self,method, timeout, data=None, headers=None, auth=None):
+        return self.exec_soap(method,timeout, data, headers, auth)
+
+    def do_request(self,method,timeout, data=None, headers=None, auth=None):
+        if settings.CIRCUIT_BREAKER:
+            return self.do_cb_request(method,timeout, data, headers, auth)
+        return self.exec_soap(method,timeout, data, headers, auth)
+
+
+    def run(self,timeout,data ,headers=None,auth=None):
+        """
+
+        :param data:
+        :param timeout:
+        :param headers:
+        :return:
+        """
+        if headers is None:
+            headers = headers
+        return self.process(self.op,self.url,timeout, data=data, headers=headers,auth=auth)
+
+    def process(self,method,url, timeout, data=None, headers=None, auth=None):
+        from zeep import Client,Transport
+        try:
+            logger.debug(
+                "Api name: " + self.name  + " url: " + str(url) + " headers:" + str(headers),
+                extra=log_json(self.halo_context))
+            now = datetime.datetime.now()
+            transport = Transport(session=self.session, timeout=timeout, operation_timeout=timeout)
+            self.client = Client(url,transport=transport)
+            soap_response = self.do_request(method,timeout, data, headers, auth)
+            total = datetime.datetime.now() - now
+            logger.info(LOGChoice.performance_data.value, extra=log_json(self.halo_context,
+                                                                         {LOGChoice.type.value: SYSTEMChoice.api.value,
+                                                                          LOGChoice.milliseconds.value: int(
+                                                                              total.total_seconds() * 1000),
+                                                                          LOGChoice.url.value: str(url)}))
+            logger.debug("ret: " + str(soap_response), extra=log_json(self.halo_context))
+            return soap_response
+        except ApiError as e:
+            msg = str(e)
+            logger.debug("error: " + msg, extra=log_json(self.halo_context))
+            raise e
+
+    def exec_soap(self,method,timeout, data=None, headers=None, auth=None):
+        if method is None:
+            raise IllegalMethodException("missing method value")
+        logger.debug("method="+str(method))
+        try:
+            soap_response = getattr(self, 'do_%s' % method)(timeout, data, headers, auth)
+            if type(soap_response) == SoapResponse:
+                return soap_response
+        except AttributeError as ex:
+            raise HaloMethodNotImplementedException("function for "+str(method),ex)
+
+from threading import RLock
+from .context import HaloContext
+lock = RLock()
+api_dict = {}
 class ApiMngr(AbsBaseClass):
-    API_LIST = []
 
+    global HALO_API_LIST
 
-    def __init__(self, halo_context):
+    def __init__1(self, halo_context):
         logger.debug("ApiMngr=" + str(halo_context))
         self.halo_context = halo_context
 
-    @staticmethod
-    def set_api_list(list):
+    def set_api_list1(self,list):
         """
 
         :param name:
@@ -341,9 +499,8 @@ class ApiMngr(AbsBaseClass):
         """
         logger.debug("set_api_list")
         if list:
-            ApiMngr.API_LIST = list
+            self.API_LIST = list
 
-    @staticmethod
     def get_api(name):
         """
 
@@ -351,17 +508,52 @@ class ApiMngr(AbsBaseClass):
         :return:
         """
         logger.debug("get_api=" + name)
-        if name in ApiMngr.API_LIST:
-            return ApiMngr.API_LIST[name]
-        return None
+        if name in HALO_API_LIST:
+            class_name = HALO_API_LIST[name]
+            if class_name:
+                return class_name
+            #else:
 
-    def get_api_instance(self, class_name, *args):
-        return Reflect.do_instantiate(__name__,class_name,AbsBaseApi,self.halo_context)
+        raise NoApiDefinitionError(name)
+
+    def get_api_instance(name, *args):
+        global api_dict
+        ctx = args[0]
+        if HaloContext.items[HaloContext.CORRELATION] in ctx.keys():
+            id = ctx.get(HaloContext.items[HaloContext.CORRELATION])
+            print("ctx:"+str(id))
+        else:
+            id = str(uuid.uuid4())[:8]
+        if name in HALO_API_LIST:
+            class_name = HALO_API_LIST[name]
+            if class_name+id in api_dict:
+                return api_dict[class_name+id]
+            else:
+                session = requests.session()
+                params = [x for x in args]
+                params.append(session)
+                api = Reflect.instantiate(class_name, AbsBaseApi, *params)
+                lock.acquire()
+                try:
+                    api_dict[class_name+id] = api
+                finally:
+                    lock.release()
+                return api
+        raise NoApiDefinitionError(name)
 
 
+    def get_api_instance2(name, *args):
+        class_name = HALO_API_LIST[name]
+        return Reflect.instantiate(class_name,AbsBaseApi,*args)
+
+    def get_api_instance1(self, class_name, *args):
+        return Reflect.instantiate(class_name,AbsBaseApi,self.halo_context)
+
+HALO_API_LIST = None
 SSM_CONFIG = None
 SSM_APP_CONFIG = None
 def load_api_config(stage_type,ssm_type,func_name,API_CONFIG):
+    global HALO_API_LIST
     global SSM_CONFIG
     global SSM_APP_CONFIG
 
@@ -391,26 +583,66 @@ def load_api_config(stage_type,ssm_type,func_name,API_CONFIG):
             logger.debug(item + ":" + url)
             for key in API_CONFIG:
                 current = API_CONFIG[key]
+                type = current["type"]
+                if type == "service":
+                    continue
                 new_url = current["url"]
                 if "service://" + item in new_url:
                     API_CONFIG[key]["url"] = new_url.replace("service://" + item, url)
     logger.debug(str(API_CONFIG))
+    api_list = {}
+    for key in API_CONFIG:
+        if "class" in API_CONFIG[key]:
+            class_name = API_CONFIG[key]["class"]
+            api_list[key] = class_name
+        else:
+            raise MissingClassConfigError(key)
 
+    HALO_API_LIST = api_list
+    #ApiMngr.instance().set_api_list(api_list)
 
+"""
+def fullname(o):
+  # o.__module__ + "." + o.__class__.__qualname__ is an example in
+  # this context of H.L. Mencken's "neat, plausible, and wrong."
+  # Python makes no guarantees as to whether the __module__ special
+  # attribute is defined, so we take a more circumspect approach.
+  # Alas, the module name is explicitly excluded from __qualname__
+  # in Python 3.
+
+  module = o.__class__.__module__
+  if module is None or module == str.__class__.__module__:
+    return o.__class__.__name__  # Avoid reporting __builtin__
+  else:
+    return module + '.' + o.__class__.__name__
+
+def get_rest_api_instance(class_name,*args):
+    ApiClass = create_api_class(class_name,(AbsRestApi, ),None)
+    instance = ApiClass(*args)
+    return instance
+
+def get_rest_api_class(name,attributes=None):
+    ApiClass = create_api_class(name,(AbsRestApi, ),attributes)
+    return "halo_flask.apis."+ApiClass.__name__
+
+def create_api_class(name,bases,attributes=None):
+    ApiClass = type(name+"Api", bases, {
+        # constructor
+        #"__init__": constructor,
+
+        # data members
+        #"string_attribute": "Geeks 4 geeks !",
+        #"int_attribute": 1706256,
+        "name" : name,
+
+        # member functions
+        #"func_arg": displayMethod,
+        #"class_func": classMethod
+    })
+    return ApiClass
+
+"""
 
 
 
 ##################################### test #########################
-
-class CnnApi(AbsBaseApi):
-    name = 'Cnn'
-
-class GoogleApi(AbsBaseApi):
-    name = 'Google'
-
-class TstApi(AbsBaseApi):
-    name = 'Tst'
-
-API_LIST = {"Google": 'GoogleApi', "Cnn": "CnnApi","Tst":"TstApi"}
-
-ApiMngr.set_api_list(API_LIST)
