@@ -14,8 +14,9 @@ from jsonpath_ng import jsonpath, parse
 # flask
 from flask import Response as HttpResponse
 
-from .utilx import Util, status
-from ..const import HTTPChoice
+from .utilx import Util
+from ..errors import status
+from ..const import HTTPChoice,ASYNC
 from ..exceptions import *
 from ..reflect import Reflect
 from ..request import HaloRequest
@@ -24,8 +25,9 @@ from ..settingsx import settingsx
 from ..logs import log_json
 from ..classes import AbsBaseClass,ServiceInfo
 from .servicex import SAGA,SEQ,FoiBusinessEvent,SagaBusinessEvent
-from ..apis import AbsBaseApi
+from ..apis import AbsBaseApi,ApiMngr
 from .filter import RequestFilter,FilterEvent
+from halo_flask.providers.providers import get_provider
 from halo_flask.saga import Saga, SagaRollBack, load_saga
 from halo_flask.models import AbsDbMixin
 
@@ -81,7 +83,7 @@ class AbsBaseMixinX(AbsBaseClass):
         ret = HaloResponse(HaloRequest(request))
         ret.payload = 'this is process get on '  # + self.get_view_name()
         ret.code = 200
-        ret.headers = []
+        ret.headers = {}
         return ret
 
     def process_post(self, request, vars):
@@ -94,7 +96,7 @@ class AbsBaseMixinX(AbsBaseClass):
         ret = HaloResponse(HaloRequest(request))
         ret.payload = 'this is process post on '  # + self.get_view_name()
         ret.code = 201
-        ret.headers = []
+        ret.headers = {}
         return ret
 
     def process_put(self, request, vars):
@@ -107,7 +109,7 @@ class AbsBaseMixinX(AbsBaseClass):
         ret = HaloResponse(HaloRequest(request))
         ret.payload = 'this is process put on '  # + self.get_view_name()
         ret.code = 200
-        ret.headers = []
+        ret.headers = {}
         return ret
 
     def process_patch(self, request, vars):
@@ -120,7 +122,7 @@ class AbsBaseMixinX(AbsBaseClass):
         ret = HaloResponse(HaloRequest(request))
         ret.payload = 'this is process patch on '  # + self.get_view_name()
         ret.code = 200
-        ret.headers = []
+        ret.headers = {}
         return ret
 
     def process_delete(self, request, vars):
@@ -133,7 +135,7 @@ class AbsBaseMixinX(AbsBaseClass):
         ret = HaloResponse(HaloRequest(request))
         ret.payload = 'this is process delete on '  # + self.get_view_name()
         ret.code = 200
-        ret.headers = []
+        ret.headers = {}
         return ret
 
     def check_author(self, request, vars, json):
@@ -159,13 +161,15 @@ class AbsBaseMixinX(AbsBaseClass):
         return True, None
 
     @abstractmethod
-    def get_dbaccess(self, halo_request):
-        return Reflect.instantiate(settings.DBACCESS_CLASS, AbsDbMixin, halo_request.context)
+    def get_dbaccess(self, halo_request,read):
+        return Reflect.instantiate(settings.DBACCESS_CLASS, AbsDbMixin, halo_request.context,read)
 
 class AbsApiMixinX(AbsBaseMixinX):
     __metaclass__ = ABCMeta
 
     business_event = None
+    secure = False
+    method_roles = None
 
     def create_response(self,halo_request, payload, headers):
         code = status.HTTP_200_OK
@@ -188,18 +192,23 @@ class AbsApiMixinX(AbsBaseMixinX):
         if foi:
             foi_name = foi["name"]
             foi_op = foi["op"]
-            instance = Reflect.instantiate(foi_name, AbsBaseApi, halo_request.context)
+            foi_conn = foi["conn"]
+            #api = ApiMngr.get_api(foi_name)
+            #instance = Reflect.instantiate(api, AbsBaseApi, halo_request.context)
+            instance = ApiMngr.get_api_instance(foi_name, halo_request.context)
             instance.op = foi_op
+            instance.conn = foi_conn
             return instance
         raise NoApiClassException("api class not defined")
 
-    def set_api_headers(self,halo_request, seq=None, dict=None):
+    def set_api_headers(self,halo_request,api, seq=None, dictx=None):
         logger.debug("in set_api_headers ")
-        if halo_request:
-            return []
+        if halo_request and api.api_type == "service":
+            return dict(halo_request.request.headers)
+        return {}
         raise HaloException("no headers")
 
-    def set_api_vars(self,halo_request, seq=None, dict=None):
+    def set_api_vars(self,halo_request,api, seq=None, dict=None):
         logger.debug("in set_api_vars " + str(halo_request))
         if True:
             ret = {}
@@ -207,10 +216,10 @@ class AbsApiMixinX(AbsBaseMixinX):
             return ret
         raise HaloException("no var")
 
-    def set_api_auth(self,halo_request, seq=None, dict=None):
+    def set_api_auth(self,halo_request,api, seq=None, dict=None):
         return None
 
-    def set_api_data(self,halo_request, seq=None, dict=None):
+    def set_api_data(self,halo_request,api, seq=None, dict=None):
         return halo_request.request.data
 
     def execute_api(self,halo_request, back_api, back_vars, back_headers, back_auth, back_data=None, seq=None, dict=None):
@@ -229,7 +238,7 @@ class AbsApiMixinX(AbsBaseMixinX):
                 raise HaloError("failed to execute api:"+str(back_api.name),e)
         return None
 
-    def extract_json(self,halo_request, back_response, seq=None):
+    def extract_json(self,halo_request,api, back_response, seq=None):
         logger.debug("in extract_json ")
         if back_response:
             try:
@@ -362,21 +371,21 @@ class AbsApiMixinX(AbsBaseMixinX):
         # 1. get api definition to access the BANK API  - url + vars dict
         back_api = getattr(self, 'set_back_api_%s' % sub_func)(halo_request)
         # 2. array to store the headers required for the API Access
-        back_headers = getattr(self, 'set_api_headers_%s' % sub_func)(halo_request)
+        back_headers = getattr(self, 'set_api_headers_%s' % sub_func)(halo_request,back_api)
         # 3. set request params
-        back_vars = getattr(self, 'set_api_vars_%s' % sub_func)(halo_request)
+        back_vars = getattr(self, 'set_api_vars_%s' % sub_func)(halo_request,back_api)
         # 4. Sset request auth
-        back_auth = getattr(self, 'set_api_auth_%s' % sub_func)(halo_request)
+        back_auth = getattr(self, 'set_api_auth_%s' % sub_func)(halo_request,back_api)
         # 5. Sset request data
         if halo_request.request.method == HTTPChoice.post.value or halo_request.request.method == HTTPChoice.put.value:
-            back_data = getattr(self, 'set_api_data_%s' % sub_func)(halo_request)
+            back_data = getattr(self, 'set_api_data_%s' % sub_func)(halo_request,back_api)
         else:
             back_data = None
         # 6. Sending the request to the BANK API with params
         back_response = getattr(self, 'execute_api_%s' % sub_func)(halo_request, back_api, back_vars,
                                                                              back_headers, back_auth, back_data)
         # 7. extract from Response stored in an object built as per the BANK API Response body JSON Structure
-        back_json = getattr(self, 'extract_json_%s' % sub_func)(halo_request, back_response)
+        back_json = getattr(self, 'extract_json_%s' % sub_func)(halo_request,back_api, back_response)
         dict = {'1': back_json}
         # 8. return json response
         return dict
@@ -396,7 +405,7 @@ class AbsApiMixinX(AbsBaseMixinX):
 
     def do_operation_3_bq(self, halo_request,sub_func):  # high maturity - saga transactions
         logger.debug("do_operation_3_bq")
-        sagax = load_saga(self.business_event.EVENT_NAME, self.business_event.saga, settings.SAGA_SCHEMA)
+        sagax = load_saga(self.business_event.EVENT_NAME,halo_request, self.business_event.saga, settings.SAGA_SCHEMA)
         payloads = {}
         apis = {}
         counter = 1
@@ -422,14 +431,14 @@ class AbsApiMixinX(AbsBaseMixinX):
 
     def do_api_work_bq(self,halo_request,sub_func, back_api, seq):
         # 3. array to store the headers required for the API Access
-        back_headers = getattr(self, 'set_api_headers_%s' % sub_func)(halo_request, seq, dict)
+        back_headers = getattr(self, 'set_api_headers_%s' % sub_func)(halo_request,back_api, seq, dict)
         # 4. set vars
-        back_vars = getattr(self, 'set_api_vars_%s' % sub_func)(halo_request, seq, dict)
+        back_vars = getattr(self, 'set_api_vars_%s' % sub_func)(halo_request,back_api, seq, dict)
         # 5. auth
-        back_auth = getattr(self, 'set_api_auth_%s' % sub_func)(halo_request, seq, dict)
+        back_auth = getattr(self, 'set_api_auth_%s' % sub_func)(halo_request,back_api, seq, dict)
         # 6. set request data
         if halo_request.request.method == HTTPChoice.post.value or halo_request.request.method == HTTPChoice.put.value:
-            back_data = getattr(self, 'set_api_data_%s' % sub_func)(halo_request, seq, dict)
+            back_data = getattr(self, 'set_api_data_%s' % sub_func)(halo_request,back_api, seq, dict)
         else:
             back_data = None
         # 7. Sending the request to the BANK API with params
@@ -437,7 +446,7 @@ class AbsApiMixinX(AbsBaseMixinX):
                                                                              back_headers, back_auth, back_data,
                                                                              seq, dict)
         # 8. extract from Response stored in an object built as per the BANK API Response body JSON Structure
-        back_json = getattr(self, 'extract_json_%s' % sub_func)(halo_request, back_response, seq)
+        back_json = getattr(self, 'extract_json_%s' % sub_func)(halo_request,back_api, back_response, seq)
         # return
         return back_json
 
@@ -467,57 +476,83 @@ class AbsApiMixinX(AbsBaseMixinX):
         # 1. get api definition to access the BANK API  - url + vars dict
         back_api = self.set_back_api(halo_request)
         # 2. array to store the headers required for the API Access
-        back_headers = self.set_api_headers(halo_request)
-        # 3. set request params
-        back_vars = self.set_api_vars(halo_request)
-        # 4. Sset request auth
-        back_auth = self.set_api_auth(halo_request)
+        back_headers = self.set_api_headers(halo_request,back_api)
+        # 3. Set request params
+        back_vars = self.set_api_vars(halo_request,back_api)
+        # 4. Set request auth
+        back_auth = self.set_api_auth(halo_request,back_api)
         # 5. Set request data
         #@todo add patch
         if halo_request.request.method == HTTPChoice.post.value or halo_request.request.method == HTTPChoice.put.value:
-            back_data = self.set_api_data(halo_request)
+            back_data = self.set_api_data(halo_request,back_api)
         else:
             back_data = None
         # 6. Sending the request to the BANK API with params
         back_response = self.execute_api(halo_request, back_api, back_vars, back_headers, back_auth,
                                                    back_data)
         # 7. extract from Response stored in an object built as per the BANK API Response body JSON Structure
-        back_json = self.extract_json(halo_request, back_response)
+        back_json = self.extract_json(halo_request,back_api, back_response)
         dict = {'1': back_json}
         # 8. return json response
         return dict
 
     def do_operation_2(self, halo_request):  # medium maturity - foi
         logger.debug("do_operation_2")
+        api_list = []
         dict = {}
         for seq in self.business_event.keys():
+            print("seq="+str(seq))
             # 1. get get first order interaction
             foi = self.business_event.get(seq)
             # 2. get api definition to access the BANK API  - url + vars dict
             back_api = self.set_back_api(halo_request, foi)
-            # 2. do api work
-            back_json = self.do_api_work(halo_request, back_api, seq, dict)
+            api_list.append(back_api)
+            if back_api.conn == ASYNC:
+                # 2.1 do async api work
+                back_json = self.do_api_async_work(halo_request, back_api, seq, dict)
+            else:
+                # 2.2 do api work
+                back_json = self.do_api_work(halo_request, back_api, seq, dict)
             # 3. store in dict
             dict[seq] = back_json
+        for api in api_list:
+            api.terminate()
         return dict
 
+    def do_api_async_work(self, halo_request, back_api, seq, dict=None):
+        # 3. array to store the headers required for the API Access
+        back_headers = self.set_api_headers(halo_request,back_api, seq, dict)
+        # 4. set vars
+        back_vars = self.set_api_vars(halo_request,back_api, seq, dict)
+        # 5. auth
+        back_auth = self.set_api_auth(halo_request,back_api, seq, dict)
+        # 6. set request data
+        if halo_request.request.method == HTTPChoice.post.value or halo_request.request.method == HTTPChoice.put.value:
+            back_data = self.set_api_data(halo_request,back_api, seq, dict)
+        else:
+            back_data = None
+        # 7. Sending the request to the BANK API with params
+        status = get_provider().add_to_queue(halo_request, back_api, back_vars, back_headers, back_auth,
+                                         back_data, seq, dict)
+        return {}
+
     def do_api_work(self,halo_request, back_api, seq, dict=None):
         # 3. array to store the headers required for the API Access
-        back_headers = self.set_api_headers(halo_request, seq, dict)
+        back_headers = self.set_api_headers(halo_request,back_api, seq, dict)
         # 4. set vars
-        back_vars = self.set_api_vars(halo_request, seq, dict)
+        back_vars = self.set_api_vars(halo_request,back_api, seq, dict)
         # 5. auth
-        back_auth = self.set_api_auth(halo_request, seq, dict)
+        back_auth = self.set_api_auth(halo_request,back_api, seq, dict)
         # 6. set request data
         if halo_request.request.method == HTTPChoice.post.value or halo_request.request.method == HTTPChoice.put.value:
-            back_data = self.set_api_data(halo_request, seq, dict)
+            back_data = self.set_api_data(halo_request,back_api, seq, dict)
         else:
             back_data = None
         # 7. Sending the request to the BANK API with params
         back_response = self.execute_api(halo_request, back_api, back_vars, back_headers, back_auth,
                                               back_data, seq, dict)
         # 8. extract from Response stored in an object built as per the BANK API Response body JSON Structure
-        back_json = self.extract_json(halo_request, back_response, seq)
+        back_json = self.extract_json(halo_request,back_api, back_response, seq)
         # return
         return back_json
 
@@ -545,7 +580,7 @@ class AbsApiMixinX(AbsBaseMixinX):
 
     def do_operation_3(self, halo_request):  # high maturity - saga transactions
         logger.debug("do_operation_3")
-        sagax = load_saga("test", self.business_event.saga, settings.SAGA_SCHEMA)
+        sagax = load_saga("test",halo_request, self.business_event.saga, settings.SAGA_SCHEMA)
         payloads = {}
         apis = {}
         counter = 1
@@ -614,35 +649,35 @@ class AbsApiMixinX(AbsBaseMixinX):
 
     def process_get(self, request, vars):
         bq = self.get_bq(vars)
-        halo_request = HaloRequest(request, bq)
+        halo_request = HaloRequest(request, bq,self.secure,self.method_roles)
         self.set_businss_event(halo_request, "x")
         ret = self.do_operation(halo_request)
         return ret
 
     def process_post(self, request, vars):
         bq = self.get_bq(vars)
-        halo_request = HaloRequest(request, bq)
+        halo_request = HaloRequest(request, bq,self.secure,self.method_roles)
         self.set_businss_event(halo_request, "x")
         ret = self.do_operation(halo_request)
         return ret
 
     def process_put(self, request, vars):
         bq = self.get_bq(vars)
-        halo_request = HaloRequest(request, bq)
+        halo_request = HaloRequest(request, bq,self.secure,self.method_roles)
         self.set_businss_event(halo_request, "x")
         ret = self.do_operation(halo_request)
         return ret
 
     def process_patch(self, request, vars):
         bq = self.get_bq(vars)
-        halo_request = HaloRequest(request, bq)
+        halo_request = HaloRequest(request, bq,self.secure,self.method_roles)
         self.set_businss_event(halo_request, "x")
         ret = self.do_operation(halo_request)
         return ret
 
     def process_delete(self, request, vars):
         bq = self.get_bq(vars)
-        halo_request = HaloRequest(request,bq)
+        halo_request = HaloRequest(request,bq,self.secure,self.method_roles)
         self.set_businss_event(halo_request, "x")
         ret = self.do_operation(halo_request)
         return ret
@@ -689,7 +724,7 @@ class PerfMixinX(AbsBaseMixinX):
         total = datetime.datetime.now() - self.now
         # return HttpResponse('performance page: timing for process: ' + str(total) + " " + str(urls) + " " + ret + " " + settings.VERSION)
         return HaloResponse(request,{"msg": 'performance page: timing for process: ' + str(total) + " " + str(
-            urls) + " " + ret + " " + settings.VERSION}, 200, [])
+            urls) + " " + ret + " " + settings.VERSION}, 200, {})
 
 
     def process_db(self, request, vars):
@@ -698,14 +733,14 @@ class PerfMixinX(AbsBaseMixinX):
         return 'db access: ' + str(total)
 
     def process_delete(self, request, vars):
-        return HaloResponse(request,{"test":"good"}, 500, [])
+        return HaloResponse(request,{"test":"good"}, 500, {})
 
 class HealthMixinX(AbsBaseMixinX):
     now = None
 
     def process_get(self, request, vars):
         self.now = datetime.datetime.now()
-        return HaloResponse(request, {"msg": 'heslth page - timing for process: ' + str(datetime.datetime.now() - self.now) + " " + settings.VERSION}, 200, [])
+        return HaloResponse(request, {"msg": 'heslth page - timing for process: ' + str(datetime.datetime.now() - self.now) + " " + settings.VERSION}, 200, {})
 
 class InfoMixinX(AbsBaseMixinX):
     now = None
@@ -716,7 +751,8 @@ class InfoMixinX(AbsBaseMixinX):
         if settings.SERVICE_INFO_CLASS:
             info = Reflect.instantiate(settings.SERVICE_INFO_CLASS,ServiceInfo)
         msg = info.toJSON()
-        return HaloResponse(request,{"data":msg, 'timing for process' : str(datetime.datetime.now() - self.now) ,"version": settings.VERSION}, 200, [])
+        msg = json.loads(msg)
+        return HaloResponse(request,{"data":msg, 'timing for process' : str(datetime.datetime.now() - self.now) ,"version": settings.VERSION}, 200, {'mimetype':"application/json"})
 
 class TestMixinX(AbsApiMixinX):
     def do_operation_1(self, halo_request):  # basic maturity - single request
