@@ -12,79 +12,11 @@ from flask import Response
 from ..settingsx import settingsx
 from halo_flask.classes import AbsBaseClass
 from halo_flask.const import HTTPChoice,LOC
-from halo_flask.request import HaloContext
+from halo_flask.context import HaloContext
 from halo_flask.exceptions import ApiTimeOutExpired, CacheError, HaloException, ProviderError
 from halo_flask.providers.providers import get_provider,ONPREM
 from halo_flask.exceptions import NoCorrelationIdException
 
-class status(AbsBaseClass):
-
-    def is_informational(code):
-        return 100 <= code <= 199
-
-    def is_success(code):
-        return 200 <= code <= 299
-
-    def is_redirect(code):
-        return 300 <= code <= 399
-
-    def is_client_error(code):
-        return 400 <= code <= 499
-
-    def is_server_error(code):
-        return 500 <= code <= 599
-
-    HTTP_100_CONTINUE = 100
-    HTTP_101_SWITCHING_PROTOCOLS = 101
-    HTTP_200_OK = 200
-    HTTP_201_CREATED = 201
-    HTTP_202_ACCEPTED = 202
-    HTTP_203_NON_AUTHORITATIVE_INFORMATION = 203
-    HTTP_204_NO_CONTENT = 204
-    HTTP_205_RESET_CONTENT = 205
-    HTTP_206_PARTIAL_CONTENT = 206
-    HTTP_207_MULTI_STATUS = 207
-    HTTP_300_MULTIPLE_CHOICES = 300
-    HTTP_301_MOVED_PERMANENTLY = 301
-    HTTP_302_FOUND = 302
-    HTTP_303_SEE_OTHER = 303
-    HTTP_304_NOT_MODIFIED = 304
-    HTTP_305_USE_PROXY = 305
-    HTTP_306_RESERVED = 306
-    HTTP_307_TEMPORARY_REDIRECT = 307
-    HTTP_308_PERMANENT_REDIRECT = 308
-    HTTP_400_BAD_REQUEST = 400
-    HTTP_401_UNAUTHORIZED = 401
-    HTTP_402_PAYMENT_REQUIRED = 402
-    HTTP_403_FORBIDDEN = 403
-    HTTP_404_NOT_FOUND = 404
-    HTTP_405_METHOD_NOT_ALLOWED = 405
-    HTTP_406_NOT_ACCEPTABLE = 406
-    HTTP_407_PROXY_AUTHENTICATION_REQUIRED = 407
-    HTTP_408_REQUEST_TIMEOUT = 408
-    HTTP_409_CONFLICT = 409
-    HTTP_410_GONE = 410
-    HTTP_411_LENGTH_REQUIRED = 411
-    HTTP_412_PRECONDITION_FAILED = 412
-    HTTP_413_REQUEST_ENTITY_TOO_LARGE = 413
-    HTTP_414_REQUEST_URI_TOO_LONG = 414
-    HTTP_415_UNSUPPORTED_MEDIA_TYPE = 415
-    HTTP_416_REQUESTED_RANGE_NOT_SATISFIABLE = 416
-    HTTP_417_EXPECTATION_FAILED = 417
-    HTTP_428_PRECONDITION_REQUIRED = 428
-    HTTP_429_TOO_MANY_REQUESTS = 429
-    HTTP_431_REQUEST_HEADER_FIELDS_TOO_LARGE = 431
-    HTTP_444_CONNECTION_CLOSED_WITHOUT_RESPONSE = 444
-    HTTP_500_INTERNAL_SERVER_ERROR = 500
-    HTTP_501_NOT_IMPLEMENTED = 501
-    HTTP_502_BAD_GATEWAY = 502
-    HTTP_503_SERVICE_UNAVAILABLE = 503
-    HTTP_504_GATEWAY_TIMEOUT = 504
-    HTTP_505_HTTP_VERSION_NOT_SUPPORTED = 505
-    HTTP_508_LOOP_DETECTED = 508
-    HTTP_510_NOT_EXTENDED = 510
-    HTTP_511_NETWORK_AUTHENTICATION_REQUIRED = 511
-
 settings = settingsx()
 
 logger = logging.getLogger(__name__)
@@ -271,14 +203,13 @@ class Util(AbsBaseClass):
     """
 
     @staticmethod
-    def json_data_response(data, status_code=200, headers=[]):
+    def json_data_response(data, status_code=200, headers={}):
         """
 
         :param data:
         :param status_code:
         :return:
         """
-        from flask import jsonify
         if status_code >= 300:
             return Response(data, status=status_code, headers=headers)
         return Response(json.dumps(data), status=status_code, headers=headers)
@@ -434,7 +365,7 @@ class Util(AbsBaseClass):
         if hasattr(e, 'detail'):
             error_detail = e.detail
         elif hasattr(e, 'original_exception'):
-            error_detail = str(e.original_exception)
+            error_detail = Util.get_detail(e.original_exception)
         else:
             if hasattr(e, 'message'):
                 e_msg = e.message
@@ -452,4 +383,13 @@ class Util(AbsBaseClass):
         if Util.isDebugEnabled(halo_context) and hasattr(e, 'stack'):
             payload["stack"] = json.dumps(e.stack)
             payload["request"] = json.dumps(Util.get_req_params(request))
-        return payload
\ No newline at end of file
+        return payload
+
+    @staticmethod
+    def get_detail(e):
+        detail = None
+        if e.original_exception:
+            detail = Util.get_detail(e.original_exception)
+        if detail:
+            return str(e)+':'+detail
+        return str(e)
\ No newline at end of file
