@@ -22,7 +22,7 @@ class HaloResponse(AbsBaseClass):
     request = None
     payload = 'this is HaloResponse'
     code = 200
-    headers = []
+    headers = {}
 
     def __init__(self,halo_request, payload=None, code=None, headers=None):
         self.request = halo_request
